@@ -9,8 +9,8 @@
 
 package org.gnu.gconf;
 
-import org.gnu.glib.Boxed;
 import org.gnu.glib.Handle;
+import org.gnu.glib.MemStruct;
 
 /**
  * Stores an entry for a GConf "directory", including a key-value pair,
@@ -18,7 +18,7 @@ import org.gnu.glib.Handle;
  * a default value, and whether GConf can write a new value at this time.
  */
 
-public class ConfEntry extends Boxed {
+public class ConfEntry extends MemStruct {
 	
 	/**
 	 * Construct a new ConfEntry object
@@ -32,54 +32,67 @@ public class ConfEntry extends Boxed {
 	public ConfEntry(Handle handle) {
 		super(handle);
 	}
+    
+    static ConfEntry getConfEntry(Handle handle) {
+        if (handle == null) {
+            return null;
+        }
+        
+        ConfEntry obj = (ConfEntry) MemStruct.getMemStructFromHandle(handle);
+        
+        if (obj == null) {
+            obj = new ConfEntry(handle);
+        }
+        
+        return obj;
+    }
 	
 	/**
 	 * Return the key field of the entry.
 	 */
 	public String getKey() {
-		return gconf_entry_get_key(handle);
+		return gconf_entry_get_key(getHandle());
 	}
 	
 	/**
 	 * Return the value field of the entry.
 	 */
 	public ConfValue getValue() {
-		Handle hndl = gconf_entry_get_value(handle);
-		return new ConfValue(hndl);
+		Handle hndl = gconf_entry_get_value(getHandle());
+		return ConfValue.getConfValue(hndl);
 	}
 	
 	public void setValue(ConfValue value) {
-		gconf_entry_set_value_nocopy(handle, value.getHandle());
+		gconf_entry_set_value_nocopy(getHandle(), value.getHandle());
 	}
 	
 	/**
 	 * Extract the value from this ConfEntry leaving the value set to null.
 	 */
 	public ConfValue stealValue() {
-		Handle hndl = gconf_entry_steal_value(handle);
-		return new ConfValue(hndl);
+		Handle hndl = gconf_entry_steal_value(getHandle());
+		return ConfValue.getConfValue(hndl);
 	}
 	
 	public String getSchemaName() {
-		return gconf_entry_get_schema_name(handle);
+		return gconf_entry_get_schema_name(getHandle());
 	}
 	
 	public void setSchemaName(String name) {
-		gconf_entry_set_schema_name(handle, name);
+		gconf_entry_set_schema_name(getHandle(), name);
 	}
 	
 	/**
 	 * Returns if the value in this entry is a default value.
 	 */
 	public boolean isDefault() {
-		return gconf_entry_get_is_default(handle);
+		return gconf_entry_get_is_default(getHandle());
 	}
 	
 	public void setIsDefault(boolean isDefault) {
-		gconf_entry_set_is_default(handle, isDefault);
+		gconf_entry_set_is_default(getHandle(), isDefault);
 	}
 
-
 	native static final protected Handle gconf_entry_new_nocopy(String key, Handle value);
 	native static final protected String gconf_entry_get_key(Handle entry);
 	native static final protected Handle gconf_entry_get_value(Handle entry);
