@@ -11,37 +11,28 @@ package org.gnu.gconf;
 
 import java.util.Date;
 
-import org.gnu.glib.Boxed;
 import org.gnu.glib.Handle;
+import org.gnu.glib.MemStruct;
 
-public class ConfMetaInfo extends Boxed {
+public class ConfMetaInfo extends MemStruct {
 	
 	public ConfMetaInfo() {
 		super(gconf_meta_info_new());
 	}
 	
 	public String getSchema() {
-		return gconf_meta_info_get_schema(handle);
+		return gconf_meta_info_get_schema(getHandle());
 	}
 	
 	public String getModUser() {
-		return gconf_meta_info_get_mod_user(handle);
+		return gconf_meta_info_get_mod_user(getHandle());
 	}
 	
 	public Date getModTime() {
-		long time = gconf_meta_info_get_mod_time(handle);
+		long time = gconf_meta_info_get_mod_time(getHandle());
 		return new Date(time);
 	}
 
-	/* (non-Javadoc)
-	 * @see java.lang.Object#finalize()
-	 */
-	protected void finalize() throws Throwable {
-		super.finalize();
-		gconf_meta_info_free(handle);
-	}
-
-
 	native static final protected Handle gconf_meta_info_new();
 	native static final protected void gconf_meta_info_free(Handle info);
 	native static final protected String gconf_meta_info_get_schema(Handle info);
