@@ -25,7 +25,7 @@ extern "C" {
 JNIEXPORT jobject JNICALL Java_org_gnu_gconf_ConfClient_gconf_1client_1get_1default
   (JNIEnv *env, jclass cls)
 {
-	return getGObjectHandle(env, (GObject *) gconf_client_get_default());
+	return getGObjectHandle(env, G_OBJECT(gconf_client_get_default()));
 }
 
 /*
@@ -36,39 +36,37 @@ JNIEXPORT jobject JNICALL Java_org_gnu_gconf_ConfClient_gconf_1client_1get_1for_
   (JNIEnv *env, jclass cls, jobject engine)
 {
 	GConfEngine* engine_g = (GConfEngine*)getPointerFromHandle(env, engine);
-	return getGObjectHandle(env, (GObject *) gconf_client_get_for_engine(engine_g));
+	return getGObjectHandle(env, G_OBJECT(gconf_client_get_for_engine(engine_g)));
 }
 
 /*
  * Class:     org_gnu_gconf_ConfClient
  * Method:    gconf_client_add_dir
  */
-JNIEXPORT void JNICALL Java_org_gnu_gconf_ConfClient_gconf_1client_1add_1dir
-  (JNIEnv *env, jclass cls, jobject client, jstring dir, jint preload, jobject exception)
+JNIEXPORT jobject JNICALL Java_org_gnu_gconf_ConfClient_gconf_1client_1add_1dir
+  (JNIEnv *env, jclass cls, jobject client, jstring dir, jint preload)
 {
 	GConfClient* client_g = (GConfClient*)getPointerFromHandle(env, client);
 	gchar *str = (gchar*)(*env)->GetStringUTFChars(env, dir, NULL);
-	GError* err;
-	err = getPointerFromHandle(env, exception);
+	GError* err = NULL;
 	gconf_client_add_dir(client_g, str, (GConfClientPreloadType)preload, &err);
-	updateHandle(env, exception, err);
 	(*env)->ReleaseStringUTFChars(env, dir, str);
+	return getStructHandle(env, err, NULL, (JGFreeFunc) g_error_free);
 }
 
 /*
  * Class:     org_gnu_gconf_ConfClient
  * Method:    gconf_client_remove_dir
  */
-JNIEXPORT void JNICALL Java_org_gnu_gconf_ConfClient_gconf_1client_1remove_1dir
-  (JNIEnv *env, jclass cls, jobject client, jstring dir, jobject exception)
+JNIEXPORT jobject JNICALL Java_org_gnu_gconf_ConfClient_gconf_1client_1remove_1dir
+  (JNIEnv *env, jclass cls, jobject client, jstring dir)
 {
 	GConfClient* client_g = (GConfClient*)getPointerFromHandle(env, client);
 	gchar *str = (gchar*)(*env)->GetStringUTFChars(env, dir, NULL);
-	GError* err;
-	err = getPointerFromHandle(env, exception);
+	GError* err = NULL;
 	gconf_client_remove_dir(client_g, str, &err);
-	updateHandle(env, exception, err);
 	(*env)->ReleaseStringUTFChars(env, dir, str);
+	return getStructHandle(env, err, NULL, (JGFreeFunc) g_error_free);
 }
 
 typedef struct 
@@ -82,7 +80,8 @@ typedef struct
 void notifyCallback(GConfClient *client, guint cnxn_id, GConfEntry *entry, gpointer data)
 {
 	NotifyData *nd = (NotifyData*)data;
-	jobject entry_g = getHandleFromPointer(nd->env, entry);
+	jobject entry_g = getStructHandle(nd->env, entry,
+			(JGCopyFunc) gconf_entry_copy, (JGFreeFunc) gconf_entry_unref);
 	(*nd->env)->CallVoidMethod(nd->env, nd->obj, nd->methodID, entry_g, nd->nameSpace);
 }
 
@@ -104,14 +103,13 @@ JNIEXPORT jint JNICALL Java_org_gnu_gconf_ConfClient_gconf_1client_1notify_1add
 	GConfClient* client_g = (GConfClient*)getPointerFromHandle(env, client);
 	static jmethodID methodID = NULL;
 	jclass cls = (*env)->GetObjectClass(env, obj);
-	GError* err;
-	err = getPointerFromHandle(env, exception);
+	GError* err = NULL;
 	gchar *str = (gchar*)(*env)->GetStringUTFChars(env, nameSpace, NULL);
 	NotifyData *data;
 	guint ret;
 	if (methodID == NULL) {
 		methodID = (*env)->GetMethodID(env, cls, "notifyCallback",
-				"(Lorg/gnu/javagnome/Handle;Ljava/lang/String;)V");
+				"(Lorg/gnu/glib/Handle;Ljava/lang/String;)V");
 		if (methodID == NULL)
 			return -1;
 	}
@@ -121,8 +119,10 @@ JNIEXPORT jint JNICALL Java_org_gnu_gconf_ConfClient_gconf_1client_1notify_1add
 	data->methodID = methodID;
 	data->nameSpace = (*env)->NewGlobalRef(env, nameSpace);
 	ret = gconf_client_notify_add(client_g, str, notifyCallback, data, notifyFree, &err);
-	updateHandle(env, exception, err);
 	(*env)->ReleaseStringUTFChars(env, nameSpace, str);
+	if (err != NULL)	{
+		updateStructHandle(env, exception, err, (JGFreeFunc) g_error_free);
+	}
 	return ret;
 }
 
@@ -191,12 +191,13 @@ JNIEXPORT jobject JNICALL Java_org_gnu_gconf_ConfClient_gconf_1client_1get
 {
 	GConfClient* client_g = (GConfClient*)getPointerFromHandle(env, client);
 	gchar *str = (gchar*)(*env)->GetStringUTFChars(env, key, NULL);
-	GError* err;
-	err = getPointerFromHandle(env, exception);
+	GError* err = NULL;
 	GConfValue* val = gconf_client_get(client_g, str, &err);
-	updateHandle(env, exception, err);
 	(*env)->ReleaseStringUTFChars(env, key, str);
-	return getHandleFromPointer(env, val);
+	if (err != NULL)	{
+		updateStructHandle(env, exception, err, (JGFreeFunc) g_error_free);
+	}
+	return getStructHandle(env, val, NULL, (JGFreeFunc) gconf_value_free);
 }
 
 /*
@@ -208,12 +209,13 @@ JNIEXPORT jobject JNICALL Java_org_gnu_gconf_ConfClient_gconf_1client_1get_1with
 {
 	GConfClient* client_g = (GConfClient*)getPointerFromHandle(env, client);
 	gchar *str = (gchar*)(*env)->GetStringUTFChars(env, key, NULL);
-	GError* err;
-	err = getPointerFromHandle(env, exception);
+	GError* err = NULL;
 	GConfValue* val = gconf_client_get_without_default(client_g, str, &err);
-	updateHandle(env, exception, err);
 	(*env)->ReleaseStringUTFChars(env, key, str);
-	return getHandleFromPointer(env, val);
+	if (err != NULL)	{
+		updateStructHandle(env, exception, err, (JGFreeFunc) g_error_free);
+	}
+	return getStructHandle(env, val, NULL, (JGFreeFunc) gconf_value_free);
 }
 
 /*
@@ -227,13 +229,14 @@ JNIEXPORT jobject JNICALL Java_org_gnu_gconf_ConfClient_gconf_1client_1get_1entr
 	GConfClient* client_g = (GConfClient*)getPointerFromHandle(env, client);
 	gchar *str1 = (gchar*)(*env)->GetStringUTFChars(env, key, NULL);
 	gchar *str2 = (gchar*)(*env)->GetStringUTFChars(env, locale, NULL);
-	GError* err;
-	err = getPointerFromHandle(env, exception);
+	GError* err = NULL;
 	GConfEntry* val = gconf_client_get_entry(client_g, str1, str2, (gboolean)schemaDefault, &err);
-	updateHandle(env, exception, err);
 	(*env)->ReleaseStringUTFChars(env, key, str1);
 	(*env)->ReleaseStringUTFChars(env, locale, str2);
-	return getHandleFromPointer(env, val);
+	if (err != NULL)	{
+		updateStructHandle(env, exception, err, (JGFreeFunc) g_error_free);
+	}
+	return getStructHandle(env, val, NULL, (JGFreeFunc) gconf_entry_unref);
 }
 
 /*
@@ -245,12 +248,13 @@ JNIEXPORT jobject JNICALL Java_org_gnu_gconf_ConfClient_gconf_1client_1get_1defa
 {
 	GConfClient* client_g = (GConfClient*)getPointerFromHandle(env, client);
 	gchar *str = (gchar*)(*env)->GetStringUTFChars(env, key, NULL);
-	GError* err;
-	err = getPointerFromHandle(env, exception);
+	GError* err = NULL;
 	GConfValue* val = gconf_client_get_default_from_schema(client_g, str, &err);
-	updateHandle(env, exception, err);
 	(*env)->ReleaseStringUTFChars(env, key, str);
-	return getHandleFromPointer(env, val);
+	if (err != NULL)	{
+		updateStructHandle(env, exception, err, (JGFreeFunc) g_error_free);
+	}
+	return getStructHandle(env, val, NULL, (JGFreeFunc) gconf_value_free);
 }
 
 /*
@@ -262,14 +266,20 @@ JNIEXPORT jboolean JNICALL Java_org_gnu_gconf_ConfClient_gconf_1client_1unset
 {
 	GConfClient* client_g = (GConfClient*)getPointerFromHandle(env, client);
 	gchar *str = (gchar*)(*env)->GetStringUTFChars(env, key, NULL);
-	GError* err;
-	err = getPointerFromHandle(env, exception);
+	GError* err = NULL;
 	gboolean val = gconf_client_unset(client_g, str, &err);
-	updateHandle(env, exception, err);
 	(*env)->ReleaseStringUTFChars(env, key, str);
+	if (err != NULL)	{
+		updateStructHandle(env, exception, err, (JGFreeFunc) g_error_free);
+	}
 	return (jboolean)val;
 }
 
+static jobject getGConfEntry(JNIEnv *env, gpointer entry)
+{
+	return getStructHandle(env, entry, NULL, (JGFreeFunc) gconf_entry_unref);
+}
+
 /*
  * Class:     org_gnu_gconf_ConfClient
  * Method:    gconf_client_all_entries
@@ -279,14 +289,15 @@ JNIEXPORT jobjectArray JNICALL Java_org_gnu_gconf_ConfClient_gconf_1client_1all_
 {
 	GConfClient* client_g = (GConfClient*)getPointerFromHandle(env, client);
 	gchar *str = (gchar*)(*env)->GetStringUTFChars(env, dir, NULL);
-	GError* err;
-	err = getPointerFromHandle(env, exception);
+	GError* err = NULL;
 	GSList* val = gconf_client_all_entries(client_g, str, &err);
-	updateHandle(env, exception, err);
 	(*env)->ReleaseStringUTFChars(env, dir, str);
+	if (err != NULL)	{
+		updateStructHandle(env, exception, err, (JGFreeFunc) g_error_free);
+	}
 	if (val == NULL)
 		return NULL;
-	return getHandleArrayFromGSList(env, val);
+	return getStructHandlesFromGSList(env, val, (GetHandleFunc) getGConfEntry);
 }
 
 /*
@@ -308,7 +319,7 @@ JNIEXPORT jobjectArray JNICALL Java_org_gnu_gconf_ConfClient_gconf_1client_1all_
 	
 	client_g = (GConfClient*)getPointerFromHandle(env, client);
 	str = (gchar*)(*env)->GetStringUTFChars(env, dir, NULL);
-	err = getPointerFromHandle(env, exception);
+	err = NULL;
 	val = gconf_client_all_dirs(client_g, str, &err);
 	(*env)->ReleaseStringUTFChars(env, dir, str);
 	if (val == NULL)
@@ -323,7 +334,9 @@ JNIEXPORT jobjectArray JNICALL Java_org_gnu_gconf_ConfClient_gconf_1client_1all_
 	 	jstring str = (*env)->NewStringUTF(env, (gchar*)item->data);
 		(*env)->SetObjectArrayElement(env, ar, i, str);
 	}
-	updateHandle(env, exception, err);
+	if (err != NULL)	{
+		updateStructHandle(env, exception, err, (JGFreeFunc) g_error_free);
+	}
 	return ar;
 }
 
@@ -331,14 +344,13 @@ JNIEXPORT jobjectArray JNICALL Java_org_gnu_gconf_ConfClient_gconf_1client_1all_
  * Class:     org_gnu_gconf_ConfClient
  * Method:    gconf_client_suggest_sync
  */
-JNIEXPORT void JNICALL Java_org_gnu_gconf_ConfClient_gconf_1client_1suggest_1sync
-  (JNIEnv *env, jclass cls, jobject client, jobject exception)
+JNIEXPORT jobject JNICALL Java_org_gnu_gconf_ConfClient_gconf_1client_1suggest_1sync
+  (JNIEnv *env, jclass cls, jobject client)
 {
 	GConfClient* client_g = (GConfClient*)getPointerFromHandle(env, client);
-	GError* err;
-	err = getPointerFromHandle(env, exception);
+	GError* err = NULL;
 	gconf_client_suggest_sync(client_g, &err);
-	updateHandle(env, exception, err);
+	return getStructHandle(env, err, NULL, (JGFreeFunc) g_error_free);
 }
 
 /*
@@ -350,11 +362,12 @@ JNIEXPORT jboolean JNICALL Java_org_gnu_gconf_ConfClient_gconf_1client_1dir_1exi
 {
 	GConfClient* client_g = (GConfClient*)getPointerFromHandle(env, client);
 	gchar *str = (gchar*)(*env)->GetStringUTFChars(env, dir, NULL);
-	GError* err;
-	err = getPointerFromHandle(env, exception);
+	GError* err = NULL;
 	gboolean val = gconf_client_dir_exists(client_g, str, &err);
-	updateHandle(env, exception, err);
 	(*env)->ReleaseStringUTFChars(env, dir, str);
+	if (err != NULL)	{
+		updateStructHandle(env, exception, err, (JGFreeFunc) g_error_free);
+	}
 	return (jboolean)val;
 }
 
@@ -367,11 +380,12 @@ JNIEXPORT jdouble JNICALL Java_org_gnu_gconf_ConfClient_gconf_1client_1get_1floa
 {
 	GConfClient* client_g = (GConfClient*)getPointerFromHandle(env, client);
 	gchar *str = (gchar*)(*env)->GetStringUTFChars(env, key, NULL);
-	GError* err;
-	err = getPointerFromHandle(env, exception);
+	GError* err = NULL;
 	gdouble val = gconf_client_get_float(client_g, str, &err);
-	updateHandle(env, exception, err);
 	(*env)->ReleaseStringUTFChars(env, key, str);
+	if (err != NULL)	{
+		updateStructHandle(env, exception, err, (JGFreeFunc) g_error_free);
+	}
 	return (jdouble)val;
 }
 
@@ -384,11 +398,12 @@ JNIEXPORT jint JNICALL Java_org_gnu_gconf_ConfClient_gconf_1client_1get_1int
 {
 	GConfClient* client_g = (GConfClient*)getPointerFromHandle(env, client);
 	gchar *str = (gchar*)(*env)->GetStringUTFChars(env, key, NULL);
-	GError* err;
-	err = getPointerFromHandle(env, exception);
+	GError* err = NULL;
 	gint val = gconf_client_get_int(client_g, str, &err);
-	updateHandle(env, exception, err);
 	(*env)->ReleaseStringUTFChars(env, key, str);
+	if (err != NULL)	{
+		updateStructHandle(env, exception, err, (JGFreeFunc) g_error_free);
+	}
 	return (jint)val;
 }
 
@@ -401,11 +416,12 @@ JNIEXPORT jstring JNICALL Java_org_gnu_gconf_ConfClient_gconf_1client_1get_1stri
 {
 	GConfClient* client_g = (GConfClient*)getPointerFromHandle(env, client);
 	gchar *str = (gchar*)(*env)->GetStringUTFChars(env, key, NULL);
-	GError* err;
-	err = getPointerFromHandle(env, exception);
+	GError* err = NULL;
 	gchar* val = gconf_client_get_string(client_g, str, &err);
-	updateHandle(env, exception, err);
 	(*env)->ReleaseStringUTFChars(env, key, str);
+	if (err != NULL)	{
+		updateStructHandle(env, exception, err, (JGFreeFunc) g_error_free);
+	}
 	return (*env)->NewStringUTF(env, val);
 }
 
@@ -418,11 +434,12 @@ JNIEXPORT jboolean JNICALL Java_org_gnu_gconf_ConfClient_gconf_1client_1get_1boo
 {
 	GConfClient* client_g = (GConfClient*)getPointerFromHandle(env, client);
 	gchar *str = (gchar*)(*env)->GetStringUTFChars(env, key, NULL);
-	GError* err;
-	err = getPointerFromHandle(env, exception);
+	GError* err = NULL;
 	gboolean val = gconf_client_get_bool(client_g, str, &err);
-	updateHandle(env, exception, err);
 	(*env)->ReleaseStringUTFChars(env, key, str);
+	if (err != NULL)	{
+		updateStructHandle(env, exception, err, (JGFreeFunc) g_error_free);
+	}
 	return (jboolean)val;
 }
 
@@ -435,12 +452,13 @@ JNIEXPORT jobject JNICALL Java_org_gnu_gconf_ConfClient_gconf_1client_1get_1sche
 {
 	GConfClient* client_g = (GConfClient*)getPointerFromHandle(env, client);
 	gchar *str = (gchar*)(*env)->GetStringUTFChars(env, key, NULL);
-	GError* err;
-	err = getPointerFromHandle(env, exception);
+	GError* err = NULL;
 	GConfSchema* val = gconf_client_get_schema(client_g, str, &err);
-	updateHandle(env, exception, err);
 	(*env)->ReleaseStringUTFChars(env, key, str);
-	return getHandleFromPointer(env, val);
+	if (err != NULL)	{
+		updateStructHandle(env, exception, err, (JGFreeFunc) g_error_free);
+	}
+	return getStructHandle(env, val, NULL, (JGFreeFunc) gconf_schema_free);
 }
 
 /*
@@ -450,22 +468,29 @@ JNIEXPORT jobject JNICALL Java_org_gnu_gconf_ConfClient_gconf_1client_1get_1sche
 JNIEXPORT jobjectArray JNICALL Java_org_gnu_gconf_ConfClient_gconf_1client_1get_1list
   (JNIEnv *env, jclass cls, jobject client, jstring key, jint type, jobject exception)
 {
-	GConfClient* client_g = (GConfClient*)getPointerFromHandle(env, client);
-	gchar *str = (gchar*)(*env)->GetStringUTFChars(env, key, NULL);
+	GConfClient* client_g;
 	GError* err;
-	err = getPointerFromHandle(env, exception);
-	GConfValueType vt = (GConfValueType)type;
-	GSList *item = NULL;
-	GSList *val = gconf_client_get_list(client_g, str, vt, &err);
-	updateHandle(env, exception, err);
+	gchar *str;
+	GConfValueType vt;
+	GSList *item;
+	GSList *val;
+	jobject schema;
+	err = NULL;
+	client_g = (GConfClient*)getPointerFromHandle(env, client);
+	str = (gchar*)(*env)->GetStringUTFChars(env, key, NULL);
+	vt = (GConfValueType)type;
+	val = gconf_client_get_list(client_g, str, vt, &err);
 	(*env)->ReleaseStringUTFChars(env, key, str);
+	if (err != NULL)	{
+		updateStructHandle(env, exception, err, (JGFreeFunc) g_error_free);
+	}
 	int i = 0;
 	jclass aCls = NULL;
 	guint len;
 	jobjectArray ar;
 	if (NULL == val)
 		return NULL;
-	len = g_slist_length(val);
+		
 	if (vt == GCONF_VALUE_STRING)
 		aCls = (*env)->FindClass(env, "java/lang/String");
 	else if (vt == GCONF_VALUE_INT)
@@ -478,9 +503,8 @@ JNIEXPORT jobjectArray JNICALL Java_org_gnu_gconf_ConfClient_gconf_1client_1get_
 		aCls = (*env)->FindClass(env, "org/gnu/gconf/ConfSchema");
 	if (NULL == aCls)
 		return NULL;
-	ar = (*env)->NewObjectArray(env, len, aCls, NULL);
-	item = val;
-	for (i = 0; item != NULL; item = item->next, ++i) {
+	ar = (*env)->NewObjectArray(env, g_slist_length(val), aCls, NULL);
+	for (item = val; item != NULL; item = item->next, ++i) {
 		if (vt == GCONF_VALUE_STRING) {
 			jstring str = (*env)->NewStringUTF(env, item->data);
 			(*env)->SetObjectArrayElement(env, ar, i, str);
@@ -510,11 +534,14 @@ JNIEXPORT jobjectArray JNICALL Java_org_gnu_gconf_ConfClient_gconf_1client_1get_
 			(*env)->SetObjectArrayElement(env, ar, i, obj);
 		}
 		else if (vt == GCONF_VALUE_SCHEMA) {
-			jmethodID mid = (*env)->GetMethodID(env, aCls, "<init>", "(I)V");
+			jmethodID mid = (*env)->GetStaticMethodID(env, aCls, "getConfSchema", "(I)V");
 			jobject obj;
 			if (mid == NULL)
 				return NULL;
-			obj = (*env)->NewObject(env, aCls, mid, (jint)item->data);
+			
+			schema = getStructHandle(env, item->data,
+					(JGCopyFunc) gconf_schema_copy, (JGFreeFunc) gconf_schema_free);
+			obj = (*env)->CallStaticObjectMethod(env, aCls, mid, schema);
 			(*env)->SetObjectArrayElement(env, ar, i, obj);
 		}
 	}
@@ -531,16 +558,17 @@ JNIEXPORT jboolean JNICALL Java_org_gnu_gconf_ConfClient_gconf_1client_1get_1pai
 {
 	GConfClient* client_g = (GConfClient*)getPointerFromHandle(env, client);
 	gchar *str = (gchar*)(*env)->GetStringUTFChars(env, key, NULL);
-	GError* err;
-	err = getPointerFromHandle(env, exception);
+	GError* err = NULL;
 	gpointer carloc = (gpointer)(*env)->GetIntArrayElements(env, car, NULL);
 	gpointer cdrloc = (gpointer)(*env)->GetIntArrayElements(env, cdr, NULL);
 	gboolean val = gconf_client_get_pair(client_g, str, (GConfValueType)carType, 
 			(GConfValueType)cdrType, carloc, cdrloc, &err);
-	updateHandle(env, exception, err);
 	(*env)->ReleaseStringUTFChars(env, key, str);
 	(*env)->ReleaseIntArrayElements(env, car, (jint*)carloc, 0);
 	(*env)->ReleaseIntArrayElements(env, cdr, (jint*)cdrloc, 0);
+	if (err != NULL)	{
+		updateStructHandle(env, exception, err, (JGFreeFunc) g_error_free);
+	}
 	return (jboolean)val;
 }
 
@@ -553,11 +581,12 @@ JNIEXPORT jboolean JNICALL Java_org_gnu_gconf_ConfClient_gconf_1client_1set_1flo
 {
 	GConfClient* client_g = (GConfClient*)getPointerFromHandle(env, client);
 	gchar *str = (gchar*)(*env)->GetStringUTFChars(env, key, NULL);
-	GError* err;
-	err = getPointerFromHandle(env, exception);
+	GError* err = NULL;
 	gboolean ret = gconf_client_set_float(client_g, str, (gdouble)val, &err);
-	updateHandle(env, exception, err);
 	(*env)->ReleaseStringUTFChars(env, key, str);
+	if (err != NULL)	{
+		updateStructHandle(env, exception, err, (JGFreeFunc) g_error_free);
+	}
 	return (jboolean)ret;
 }
 
@@ -570,11 +599,12 @@ JNIEXPORT jboolean JNICALL Java_org_gnu_gconf_ConfClient_gconf_1client_1set_1int
 {
 	GConfClient* client_g = (GConfClient*)getPointerFromHandle(env, client);
 	gchar *str = (gchar*)(*env)->GetStringUTFChars(env, key, NULL);
-	GError* err;
-	err = getPointerFromHandle(env, exception);
+	GError* err = NULL;
 	gboolean ret = gconf_client_set_int(client_g, str, (gint)val, &err);
-	updateHandle(env, exception, err);
 	(*env)->ReleaseStringUTFChars(env, key, str);
+	if (err != NULL)	{
+		updateStructHandle(env, exception, err, (JGFreeFunc) g_error_free);
+	}
 	return (jboolean)ret;
 }
 
@@ -588,12 +618,13 @@ JNIEXPORT jboolean JNICALL Java_org_gnu_gconf_ConfClient_gconf_1client_1set_1str
 	GConfClient* client_g = (GConfClient*)getPointerFromHandle(env, client);
 	gchar *str1 = (gchar*)(*env)->GetStringUTFChars(env, key, NULL);
 	gchar *str2 = (gchar*)(*env)->GetStringUTFChars(env, val, NULL);
-	GError* err;
-	err = getPointerFromHandle(env, exception);
+	GError* err = NULL;
 	gboolean ret = gconf_client_set_string(client_g, str1, str2, &err);
-	updateHandle(env, exception, err);
 	(*env)->ReleaseStringUTFChars(env, key, str1);
 	(*env)->ReleaseStringUTFChars(env, key, str2);
+	if (err != NULL)	{
+		updateStructHandle(env, exception, err, (JGFreeFunc) g_error_free);
+	}
 	return (jboolean)ret;
 }
 
@@ -606,11 +637,12 @@ JNIEXPORT jboolean JNICALL Java_org_gnu_gconf_ConfClient_gconf_1client_1set_1boo
 {
 	GConfClient* client_g = (GConfClient*)getPointerFromHandle(env, client);
 	gchar *str = (gchar*)(*env)->GetStringUTFChars(env, key, NULL);
-	GError* err;
-	err = getPointerFromHandle(env, exception);
+	GError* err = NULL;
 	gboolean ret = gconf_client_set_bool(client_g, str, (gboolean)val, &err);
-	updateHandle(env, exception, err);
 	(*env)->ReleaseStringUTFChars(env, key, str);
+	if (err != NULL)	{
+		updateStructHandle(env, exception, err, (JGFreeFunc) g_error_free);
+	}
 	return (jboolean)ret;
 }
 
@@ -624,11 +656,12 @@ JNIEXPORT jboolean JNICALL Java_org_gnu_gconf_ConfClient_gconf_1client_1set_1sch
 	GConfClient* client_g = (GConfClient*)getPointerFromHandle(env, client);
 	gchar *str = (gchar*)(*env)->GetStringUTFChars(env, key, NULL);
 	GConfSchema* val_g = (GConfSchema*)getPointerFromHandle(env, val);
-	GError* err;
-	err = getPointerFromHandle(env, exception);
+	GError* err = NULL;
 	gboolean ret = gconf_client_set_schema(client_g, str, val_g, &err);
-	updateHandle(env, exception, err);
 	(*env)->ReleaseStringUTFChars(env, key, str);
+	if (err != NULL)	{
+		updateStructHandle(env, exception, err, (JGFreeFunc) g_error_free);
+	}
 	return (jboolean)ret;
 }
 
@@ -641,8 +674,7 @@ JNIEXPORT jboolean JNICALL Java_org_gnu_gconf_ConfClient_gconf_1client_1set_1lis
 {
 	GConfClient* client_g = (GConfClient*)getPointerFromHandle(env, client);
 	gchar *str = (gchar*)(*env)->GetStringUTFChars(env, key, NULL);
-	GError* err;
-	err = getPointerFromHandle(env, exception);
+	GError* err = NULL;
 	GConfValueType vt = (GConfValueType)type;
 	int i = 0;
 	jclass aCls = NULL;
@@ -700,9 +732,11 @@ JNIEXPORT jboolean JNICALL Java_org_gnu_gconf_ConfClient_gconf_1client_1set_1lis
 		}
 		gslist = g_slist_append(gslist, data);
 	}
-	ret = gconf_client_set_list(client_g, str, vt, gslist, &err);
-	updateHandle(env, exception, err);
+	ret = gconf_client_set_list(client_g, str, vt, gslist, &err);;
 	(*env)->ReleaseStringUTFChars(env, key, str);
+	if (err != NULL)	{
+		updateStructHandle(env, exception, err, (JGFreeFunc) g_error_free);
+	}
 	return (jboolean)ret;
 }
 
@@ -716,12 +750,13 @@ JNIEXPORT jboolean JNICALL Java_org_gnu_gconf_ConfClient_gconf_1client_1set_1pai
 {
 	GConfClient* client_g = (GConfClient*)getPointerFromHandle(env, client);
 	gchar *str = (gchar*)(*env)->GetStringUTFChars(env, key, NULL);
-	GError* err;
-	err = getPointerFromHandle(env, exception);
+	GError* err = NULL;
 	gboolean ret = gconf_client_set_pair(client_g, str, (GConfValueType)carType, 
 			(GConfValueType)cdrType, (gconstpointer)car, (gconstpointer)cdr, &err);
-	updateHandle(env, exception, err);
 	(*env)->ReleaseStringUTFChars(env, key, str);
+	if (err != NULL)	{
+		updateStructHandle(env, exception, err, (JGFreeFunc) g_error_free);
+	}
 	return (jboolean)ret;
 }
 
